@@ -1,7 +1,7 @@
 /** @file  mset.h
  *  @brief Class representing a list of search results
  */
-/* Copyright (C) 2015,2016 Olly Betts
+/* Copyright (C) 2015,2016,2019 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -65,6 +65,14 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      */
     MSet & operator=(const MSet & o);
 
+#ifdef XAPIAN_MOVE_SEMANTICS
+    /// Move constructor.
+    MSet(MSet && o);
+
+    /// Move assignment operator.
+    MSet & operator=(MSet && o);
+#endif
+
     /** Default constructor.
      *
      *  Creates an empty MSet, mostly useful as a placeholder.
@@ -171,24 +179,50 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 *  was found in text. If not enabled, snippet() returns a (sub)string
 	 *  of text without any highlighted terms.
 	 */
-	SNIPPET_EMPTY_WITHOUT_MATCH = 4
+	SNIPPET_EMPTY_WITHOUT_MATCH = 4,
+
+	/** Enable generation of n-grams from CJK text.
+	 *
+	 *  This option highlights CJK searches made using the QueryParser
+	 *  FLAG_CJK_NGRAM flag.  Non-CJK characters are split into words as
+	 *  normal.
+	 *
+	 *  The TermGenerator FLAG_CJK_NGRAM flag needs to have been used at
+	 *  index time.
+	 *
+	 *  This mode can also be enabled by setting environment variable
+	 *  XAPIAN_CJK_NGRAM to a non-empty value (but doing so was deprecated
+	 *  in 1.4.11).
+	 *
+	 *  @since Added in Xapian 1.4.11.
+	 */
+	SNIPPET_CJK_NGRAM = 2048
     };
 
     /** Generate a snippet.
      *
-     *  This method selects a continuous run of words of less than about @a
-     *  length bytes from @a text, based mainly on where the query matches
-     *  (currently terms, exact phrases and wildcards are taken into account),
-     *  but also on the non-query terms in the text.
+     *  This method selects a continuous run of words from @a text, based
+     *  mainly on where the query matches (currently terms, exact phrases and
+     *  wildcards are taken into account).  If flag SNIPPET_BACKGROUND_MODEL is
+     *  used (which it is by default) then the selection algorithm also
+     *  considers the non-query terms in the text with the aim of showing
+     *  a context which provides more useful information.
      *
-     *  The returned text can be escaped (by default, it is escaped to make it
-     *  suitable for use in HTML), and matches with the query will be
+     *  The size of the text selected can be controlled by the @a length
+     *  parameter, which specifies a number of bytes of text to aim to select.
+     *  However slightly more text may be selected.  Also the size of any
+     *  escaping, highlighting or omission markers is not considered.
+     *
+     *  The returned text is escaped to make it suitable for use in HTML
+     *  (though beware that in upstream releases 1.4.5 and earlier this
+     *  escaping was sometimes incomplete), and matches with the query will be
      *  highlighted using @a hi_start and @a hi_end.
      *
      *  If the snippet seems to start or end mid-sentence, then @a omit is
      *  prepended or append (respectively) to indicate this.
      *
-     *  The stemmer used to build the query should be specified in @a stemmer.
+     *  The same stemming algorithm which was used to build the query should be
+     *  specified in @a stemmer.
      *
      *  And @a flags contains flags controlling behaviour.
      *
@@ -204,7 +238,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 
     /** Prefetch hint a range of items.
      *
-     *  For a remote database, this may start a pipelined fetched of the
+     *  For a remote database, this may start a pipelined fetch of the
      *  requested documents from the remote server.
      *
      *  For a disk-based database, this may send prefetch hints to the
@@ -216,7 +250,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 
     /** Prefetch hint a single MSet item.
      *
-     *  For a remote database, this may start a pipelined fetched of the
+     *  For a remote database, this may start a pipelined fetch of the
      *  requested documents from the remote server.
      *
      *  For a disk-based database, this may send prefetch hints to the
@@ -228,7 +262,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 
     /** Prefetch hint the whole MSet.
      *
-     *  For a remote database, this may start a pipelined fetched of the
+     *  For a remote database, this may start a pipelined fetch of the
      *  requested documents from the remote server.
      *
      *  For a disk-based database, this may send prefetch hints to the
@@ -454,6 +488,14 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
      */
     Xapian::doccount get_collapse_count() const;
 
+    /** Return the sort key for the current position.
+     *
+     *  If sorting didn't use a key then an empty string will be returned.
+     *
+     *  @since Added in Xapian 1.4.6.
+     */
+    std::string get_sort_key() const;
+
     /** Convert the weight of the current iterator position to a percentage.
      *
      *  The matching document with the highest weight will get 100% if it
